/*-
 * Copyright (c) 2000 Chris Costello <chris@FreeBSD.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <err.h>
#include <string.h>
#include <stdio.h>

#undef __FreeBSD__
#if defined(__FreeBSD__)
extern char	*__progname;
#endif /* __FreeBSD__ */

static char	*pname;

void
usage()
{
	fprintf(stderr, "usage: %s [input [output]]\n", pname);
	exit(1);
}

int
main(argc, argv)
	int argc;
	char *argv[];
{
	FILE *fin, *fout;
	int c;
	char *infile, *outfile;

	outfile = infile = NULL;
	fin = fout = NULL;

#if defined(__FreeBSD__)
	pname = __progname;
#else
	pname = strchr(argv[0], '/');
	if (pname == NULL)
		pname = argv[0];
	else
		pname++;
#endif /* __FreeBSD__ */

	switch (argc) {
	case 3:
		fout = fopen((outfile = argv[2]), "w");
		if (fout == NULL)
			err(1, "%s", outfile);
		/* FALLTHROUGH */
	case 2:
		fin = fopen((infile = argv[1]), "r");
		if (fin == NULL)
			err(1, "%s", infile);
		/* FALLTHROUGH */
	case 1:
		break;
	default:
		usage();
		/* NOTREACHED */
	}

	if (outfile == NULL) {
		outfile = "stdout";
		fout = stdout;
	}
	if (infile == NULL) {
		infile = "stdin";
		fin = stdin;
	}

	if (strcmp(pname, "dos2bsd") == 0) {
		while ((c = fgetc(fin)) != EOF) {
			if (c == '\r') {
				c = fgetc(fin);
				if (c == '\n')
					fputc(c, fout);
				else {
					fputc('\r', fout);
					ungetc(c, fin);
				}
			} else fputc(c, fout);
		}
	} else if (strcmp(pname, "bsd2dos") == 0) {
		while ((c = fgetc(fin)) != EOF) {
			if (c == '\n')
				fputc('\r', fout);
			fputc(c, fout);
		}
	} else if (strcmp(pname, "mac2bsd") == 0) {
		while ((c = fgetc(fin)) != EOF) {
			if (c == '\r')
				fputc('\n', fout);
			else
				fputc(c, fout);
		}
	} else if (strcmp(pname, "bsd2mac") == 0) {
		while ((c = fgetc(fin)) != EOF) {
			if (c == '\n')
				fputc('\r', fout);
			else
				fputc(c, fout);
		}
	} else errx(1, "must be run as bsd2dos, dos2bsd, mac2bsd or bsd2mac");

	if (ferror(fin))
		warn("%s", infile);
	if (ferror(fout))
		warn("%s", outfile);
	fclose(fin);
	fclose(fout);

	return 0;
}
